/* --- LIBRARIES --- */
#include <Servo.h>
#include <MK2.h>

// Wheels
MotorController motorController(3,2,5,4);
//Shoulders
ServoMotor leftShoulder(22,true);
ServoMotor rightShoulder(38);
//Elbows
ServoMotor leftElbow(24,true);
ServoMotor rightElbow(40);
//Arms
StepperMotor leftArm(A3,A2,A4,3600);
StepperMotor rightArm(A6,A5,A7,3600);
//Left Grippers
ServoMotor L1(26);
ServoMotor L2(28);
ServoMotor L3(30);
ServoMotor L4(32);
ServoMotor L5(34);
//Right Grippers
ServoMotor R1(42);
ServoMotor R2(44);
ServoMotor R3(46);
ServoMotor R4(48);
ServoMotor R5(50);
//RangeFinders
RangeFinder upperLeft(7,6);
RangeFinder upperRight(13,12);
//Fans
Fans fans(A1);

//USB Data Variables
String USBStr = "";
int fromUSBData[18] = { 0,0, //leftWheel,rightWheel
                        90,90, //leftShoulder,rightShoulder 
                        90,90, //leftElbow,rightElbow
                        0,0, //leftArm,rightArm
                        90,90,90,90,90, //L0,L1,L2,L3,L4
                        90,90,90,90,90 }; //R0,R1,R2,R3,R4
int toUSBData[3] = { 0,0 }; //upperLeftRF, upperRightRF;
int USBIndex = 0;
char USBChr = ' ';

//Data sending delay
unsigned long startDelay = 10000; 
unsigned long endTime = 0;
float delayTime = 30;

/* ----- SETUP ----- */
void setup() {
  //Begin USB Serial
  Serial.begin(57600);
  
  //Turn motor controller on
  motorController.initialize();
  //Turn shoulders on
  leftShoulder.on();
  rightShoulder.on();
  //Turn elbows on
  leftElbow.on();
  rightElbow.on();
  //Home arms
  leftArm.initialize();
  rightArm.initialize();
  leftArm.home();
  rightArm.home();
  //Turn left grippers on
  L1.on();
  L2.on();
  L3.on();
  L4.on();
  L5.on();
  //Turn right grippers on
  R1.on();
  R2.on();
  R3.on();
  R4.on();
  R5.on();
  //Turn RangeFinders on
  upperLeft.initialize();
  upperRight.initialize();
  //Turn fans on
  fans.initialize();
  fans.on();
}

/* --- MAIN lOOP --- */
void loop() {
  //Read from USB
  while(Serial.available()) {       
    USBChr = Serial.read(); 
    parseData(USBChr, USBStr, fromUSBData, USBIndex); 
  }
  
  //Update All
  updateAll();

  //wait 5 seconds before sending data
  unsigned long t = millis();
  if(t > startDelay) {
    if(t > endTime) {
      //increment the time
      endTime = t + delayTime;
      //send data
      //toUSBData[0] = upperLeft.getRange();
      //toUSBData[1] = upperRight.getRange();
      //sendData(toUSBData, 2);
      //flush data
      Serial.flush();
    }
  }
}

/* --- HELPER FUNCTIONS --- */

//Parses data
void parseData(char chr, String& dataStr, int data[], int& dataIndex ) {
  //begin parsing
  if(chr == '{') { dataStr = ""; dataIndex = 0; }
  //extract number from data
  else if(chr == ',' || chr == '}') {
    data[dataIndex] = dataStr.toInt();
    dataIndex++; dataStr = "";
    //Finished reading message
    if(chr == '}') { dataStr = ""; dataIndex = 0; }   
  }
  //add char to current string
  else if (!isWhitespace(chr) && chr != '\n'){ dataStr += chr; }
}
//Sends data
void sendData(int data[], int dataLen) {
  String toStr = "";
  if(dataLen == 1) {
    toStr = "{ " + String(data[0]) + " }\n";
  } else if (dataLen > 1) {
    toStr = "{ ";
    for(int i= 0; i < dataLen; i++) { 
      toStr += String(data[i]); 
      if(i < dataLen-1) { toStr += ", "; }
    }
    toStr += " }\n";
  }
  Serial.print(toStr);
}

//Updates all joints
void updateAll() {  
  //update range finders
  upperLeft.update();
  upperRight.update();
  //Update wheels
  motorController.setVelocities(fromUSBData[0],fromUSBData[1]);
  motorController.update();
  //update shoulders
  leftShoulder.rotateTo(fromUSBData[2]); leftShoulder.update();
  rightShoulder.rotateTo(fromUSBData[3]); rightShoulder.update();
  //update elbows
  leftElbow.rotateTo(fromUSBData[4]); leftElbow.update();
  rightElbow.rotateTo(fromUSBData[5]); rightElbow.update();
  //update arms
  leftArm.stepTo(fromUSBData[6]); leftArm.update();
  rightArm.stepTo(fromUSBData[7]); rightArm.update();  
  //update left grippers
  L1.rotateTo(fromUSBData[8]); L1.update();
  L2.rotateTo(fromUSBData[9]); L2.update();
  L3.rotateTo(fromUSBData[10]); L3.update();
  L4.rotateTo(fromUSBData[11]); L4.update();
  L5.rotateTo(fromUSBData[12]); L5.update();
  //update right grippers
  R1.rotateTo(fromUSBData[14]); R1.update();
  R2.rotateTo(fromUSBData[15]); R2.update();
  R3.rotateTo(fromUSBData[16]); R3.update();
  R4.rotateTo(fromUSBData[17]); R4.update();
  R5.rotateTo(fromUSBData[18]); R5.update();
}


